import unittest
from ssmcommander.yaml_parser import InputFile
import os

from moto import mock_kms


class MyTestCase(unittest.TestCase):
    @mock_kms
    def test_bad_kms_block(self):
        # mocked kms decrypt is just double base64'ed
        input = os.path.join(os.path.dirname(os.path.abspath(__file__)), "test_badkms.yml")
        file = InputFile(input)
        self.assertIsNotNone(input)
        self.assertEqual(file.raw_data['section_bad_kms_value']['password'], "boblawblob")

    def test_can_read(self):
        input = os.path.join(os.path.dirname(os.path.abspath(__file__)), "test.yml")
        self.assertIsNotNone(input)


if __name__ == '__main__':
    unittest.main()



import treelib
from treelib.exceptions import DuplicatedNodeIdError
import yaml

path_separator = '/'
secure_string_identifier = '[*]'


class Tree:
    def __init__(self, prefix, session):
        self.session = session

        if prefix[0] != "/":
            prefix = "/" + prefix
        self.prefix = prefix
        self.tree = treelib.Tree()

    def get_tree_from_path(self, path=None):
        path = path.split(path_separator)
        node_list = []
        for index, node in enumerate(path):
            if node:
                node_id = "/".join(path[0:index + 1])
                node_list.append({'node': node, 'id': node_id})
        for index, node in enumerate(path):
            if index == 1:
                node_list[index - 1]['parent'] = None
            else:
                node_list[index - 1]['parent'] = node_list[index - 2]['id']
        return node_list

    def fetch_from_ssm(self, with_decryption=False):
        client = self.session.client("ssm")
        paginator = client.get_paginator('get_parameters_by_path')
        pages = paginator.paginate(
            Path=self.prefix,
            Recursive=True,
            WithDecryption=with_decryption
        )
        ssm_params = []
        for page in pages:
            parameters_page = [{"name": entry['Name'],
                                "type": entry['Type'],
                                "value": entry['Value'],
                                "version": entry['Version']} for entry in page['Parameters']]
            ssm_params.extend(parameters_page)

        # build the tree from the params
        for item in ssm_params:
            for node in self.get_tree_from_path(item['name']):
                try:
                    self.tree.create_node(node['node'], node['id'], parent=node['parent'], data=item)
                except DuplicatedNodeIdError:
                    pass

    def echo(self):
        self.tree.show()

    def dump(self):
        prefix_id = self.prefix[:]
        if prefix_id[-1] == "/":
            prefix_id = prefix_id[:-1]

        # find node of prefix
        root_tree = self.tree.subtree(prefix_id)

        data = {}
        for node in root_tree.leaves():
            parts = node.identifier[len(prefix_id):].split("/")[1:]

            pointer = data
            index = 0
            for p in parts:
                if p not in pointer:
                    pointer[p] = {}

                if index == len(parts) - 1:
                    pointer[p] = node.data['value']
                else:
                    pointer = pointer[p]
                index += 1

        print(yaml.dump(data))



import click
from ssmcommander.tree import Tree
from ssmcommander.yaml_parser import InputFile
from ssmcommander import CommanderSession
import logging

log = logging.getLogger()


def get_session(region=None, ssm_override_url=None, kms_override_url=None):
    overrides = {}
    if ssm_override_url:
        overrides["ssm"] = ssm_override_url
    if kms_override_url:
        overrides["kms"] = kms_override_url

    return CommanderSession(region_name=region, endpoint_overrides=overrides)


@click.group()
@click.option('--debug', prompt=False, is_flag=True)
def cli(debug):
    if debug:
        log.setLevel(logging.DEBUG)

        handler = logging.StreamHandler()
        handler.setLevel(logging.DEBUG)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        handler.setFormatter(formatter)
        log.addHandler(handler)

        log.debug("debug logging on")
    pass


@click.command()
@click.option('--region', prompt=False)
@click.option('--ssm-override-url', prompt=False)
@click.argument('ssm_prefix')
def dump(region, ssm_prefix, ssm_override_url=None):
    """Dump the values in ssm to a param file, for backup or inspection.  Suitable for sending in to the write command"""
    session = get_session(region, ssm_override_url=ssm_override_url)
    tree = Tree(ssm_prefix, session)
    tree.fetch_from_ssm(with_decryption=True)
    tree.dump()


@click.command("tree")
@click.option('--ssm-override-url', prompt=False)
@click.option('--region', prompt=False)
@click.argument('ssm_prefix')
def tree_func(region, ssm_prefix, ssm_override_url=None):
    """Print out a tree of your SSM parameters"""
    session = get_session(region, ssm_override_url=ssm_override_url)
    tree = Tree(ssm_prefix, session)
    tree.fetch_from_ssm()
    tree.echo()


@click.command()
@click.option('--region', prompt=False)
@click.option('--ssm-override-url', prompt=False)
@click.option('--kms-override-url', prompt=False)
@click.option('--overwrite/--no-overwrite', default=False, prompt=False)
@click.argument('ssm_prefix')
@click.argument('file')
def write(region, ssm_prefix, file, overwrite, ssm_override_url=None, kms_override_url=None):
    """take a yaml file, put it in to SSM"""
    parsed_file = InputFile(file)
    session = get_session(region, ssm_override_url=ssm_override_url, kms_override_url=kms_override_url)
    parsed_file.write_to_ssm(session, ssm_prefix, overwrite=overwrite)


cli.add_command(dump)
cli.add_command(tree_func)
cli.add_command(write)

if __name__ == "__main__":
    cli()



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='aws-ssm-commander',
    version='0.0.4',
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    include_package_data=True,
    long_description=long_description,
    description="A utility for dealing with AWS SSM Parameter Store",
    long_description_content_type="text/markdown",
    url="https://github.com/djcrabhat/aws-ssm-commander",
    install_requires=[
        'Click',
        'boto3',
        'pyyaml',
        'treelib'
    ],
    entry_points='''
        [console_scripts]
        aws-ssm-commander=ssmcommander.cli:cli
    ''',
)



from click import open_file, secho
from click.exceptions import BadParameter
import yaml
import treelib
import logging
import boto3
import base64
from botocore.exceptions import ClientError
import binascii

log = logging.getLogger()

# TODO: understand this loader nonsense better
loader = yaml.SafeLoader


class InputFile:
    def __init__(self, file, kms_session=None):
        if kms_session:
            self.kms_session = kms_session
        else:
            self.kms_session = boto3.Session()

        yaml.add_constructor(u'!kms', self.kms_ctor, yaml.SafeLoader)

        with open_file(file) as f:
            self.raw_data = yaml.load(f, loader)

        self.build_config_tree()

    def kms_ctor(self, loader, node):
        try:
            binary_data = base64.b64decode(node.value)

            if self.kms_session is None:
                raise BadParameter("could not establish a KMS session")
            kms = self.kms_session.client('kms')
            meta = kms.decrypt(CiphertextBlob=binary_data)

            unencrypted = meta[u'Plaintext'].decode()
            return unencrypted
        except ClientError as ex:
            secho(ex.response['Error']['Code'])
            raise BadParameter("could not decode !kms value: {}".format(ex))
        except binascii.Error as ex:
            secho("Cannot parse b64 blob: {}".format(ex))
            raise BadParameter("could not decode !kms value: {}".format(ex))

    def build_config_tree(self):
        self.input_tree = treelib.Tree()
        self.input_tree.create_node("{root}", "{root}")
        self._build_tree(self.input_tree, self.raw_data, "{root}")

    def _build_tree(self, tree, data, parent):
        if isinstance(data, dict):
            for key, value in data.items():
                if parent:
                    key_id = "/".join([parent, key])
                else:
                    key_id = key

                is_secure_string = False
                secure_string_keys = ["value", "kms_key_id"]
                if isinstance(value, dict):
                    if all(key in value.keys() for key in secure_string_keys):
                        is_secure_string = True

                if isinstance(value, str) or is_secure_string:
                    tree.create_node(key, key_id, parent=parent, data=value)
                else:
                    tree.create_node(key, key_id, parent=parent)
                    self._build_tree(tree, value, key_id)

    def write_to_ssm(self, session, prefix, show=True, overwrite=False):
        if show:
            self.input_tree.show()

        if prefix[0] != "/":
            prefix = "/" + prefix
        if prefix[-1] == "/":
            prefix = prefix[:-1]

        param_count = 0
        leaves = self.input_tree.leaves()

        client = session.client('ssm')
        for leaf in leaves:
            ssm_path = leaf.identifier.format(root=prefix)
            try:
                key_id = None
                description = None
                if isinstance(leaf.data, str):
                    value = leaf.data
                    type = 'String'
                elif isinstance(leaf.data, dict):
                    value = leaf.data['value']
                    key_id = leaf.data['kms_key_id']
                    type = 'SecureString'
                elif isinstance(leaf.data, list):
                    value = ",".join(leaf.data)
                    type = 'StringList'
                else:
                    raise ValueError("{} was not a string, list or dictionary, {}".format(ssm_path, leaf.data))
                params = dict(Name=ssm_path,
                              Value=value,
                              Type=type,
                              Overwrite=overwrite)
                if key_id:
                    params['KeyId'] = key_id
                if description:
                    params['Description'] = description

                log.debug("writing to {}".format(ssm_path))
                client.put_parameter(**params)
                param_count += 1
            except Exception as ex:
                log.exception("error writing '{}' to ssm".format(leaf.tag))
                secho("cannot write to SSM: {}".format(ex), fg='red')

        secho("Wrote {} params to SSM on path {}".format(param_count, prefix))



import boto3


class CommanderSession:
    def __init__(self, endpoint_overrides=None, region_name=None):
        self.endpoint_override = {}
        if endpoint_overrides:
            self.endpoint_override.update(endpoint_overrides)
        params = {}
        if region_name:
            params["region_name"] = region_name

        self.session = boto3.Session(**params)

    def client(self, service, **kwargs):
        params = {}
        if service in self.endpoint_override:
            params["endpoint_url"] = self.endpoint_override[service]

        if kwargs:
            params.update(kwargs)

        client = self.session.client("ssm", **params)
        return client
